#!/usr/local/bin/perl -wT

# iVote - A visual poll and voting system
# (C) Copyright 2001 by Michael Holve
# Released under the GNU license
#
# v0.1    - March 21, 2000
# v0.2    - March 22, 2000
# v0.03   - July 24, 2000
# v0.04   - January 24, 2001
# v0.05   - January 29, 2001

use strict;
use vars qw($dbh);
use Apache::DBI;
use Time::localtime;
use CGI qw(:standard);
#
# Uncomment to send errors to browser rather than Apache error_log
#
use CGI::Carp qw(fatalsToBrowser);
#
# Just some precautions here to thwart malicious folks
#
$CGI::POST_MAX=1024 * 100;  # Maximum of 100K posts
$CGI::DISABLE_UPLOADS = 1;  # No uploads if true(1)
#
# Our version number - don't change this...
#
my $version = "v0.05";


# Begin User Configuration Section
#
my $home            = "/iVote";
my $images          = "$home/images";
my $buttons         = "$home/buttons";
my $cgi_base        = "/perl";
my $cgi_url         = "$cgi_base/iVote.pl";
my $file_path       = "/full/path/to/iVote";
my $db_name         = "ivote";
my $db_votes        = "voted";
my $db_entry        = "entries";
my $header_doc      = "$file_path/header.html";
my $footer_doc      = "$file_path/footer.html";
my $dump_page_title = "Welcome to iVote! ($version)";
my $dump_page_attrs = 'BGCOLOR = "#FFFFFF" TEXT="#000000"';
my $image_type      = "jpg";
my $rank_next       = "http://localhost/$cgi_url";
#
# Options, limits and toggles
#
my $singlevote      = 0;  # 0 = multiple, 1 = single
my $allowdelete     = 1;  # 0 = no, 1 = yes
my $cutoff          = 0;  # 0 = allow, 1 = disallow, "2000-7-24" would set date
my $vote_limit      = 0;  # Max change in vote value (0 = normal, 1 = +/-, 2 = --/++)
my $ivote_down      = 0;  # 0 = up, 1 = down
my $mailto_name     = 1;  # 0 = no link, 1 = link
my $blurb           = 1;  # 0 = no blurb shown, 1 = show blurb
my $max_rank        = 5;  # Max number of listings in ranking display
my $rank_head       = 1;  # 0 = no header, 1 = show header file
my $vote_style      = 0;  # 0 = default, 1 = clean (with "rankings" finish)
#
# End User Configuration Section
#
# DO NOT CHANGE ANYTHING BELOW THIS LINE, IT SHOULDN'T BE NECESSARY!


# WebHeader - Pipe the $header_doc into the top of pages
#
sub WebHeader  {

  print "</HEAD>\n<BODY $dump_page_attrs>\n\n";
  open HEADER, $header_doc ||
    die "Cannot open header file $header_doc\n";
  while( <HEADER> )  {
   print $_;
  }
  close HEADER;
  print "\n<CENTER>\n\n";
}


# WebFooter - Print copyright information at the bottom of the dynamic page and
# then include the user-supplied footer page.  DO NOT DISABLE THIS ROUTINE!
#
sub WebFooter  {

  print "<CENTER><B>\n<A HREF=\"http://eunuchs.org/linux/ivote/index.html\">iVote</A> is Copyright\n";
  print "&copy 2001 by <A HREF=\"mailto:mike\@eunuchs.org\">Michael Holve</A>\n</B></CENTER>\n\n";

  open FOOTER, $footer_doc ||
    die "Cannot open footer file $footer_doc\n";
  while( <FOOTER> )  {
   print $_;
  }
  close FOOTER;
}


# NoDeleteReset - Warn that deletes and resets are disabled
#
sub NoDeleteReset  {

  &WebHeader;
  print "<H1>Reset and delete are disabled!</H1>\n";
  print "<A HREF=\"$cgi_url?command=admin\"><IMG SRC=\"$buttons/okay.png\" WIDTH=102 HEIGHT=24 BORDER=0></A><BR><BR>\n";
  &WebFooter;
}


# ZapDB - takes an argument for the table to clear out (zap)
#
sub ZapDB  {

  &WebHeader;
  print "<BR>\n<TABLE BORDER=\"1\" CELLPADDING=\"4\" CELLSPACING=\"0\">
  <TR><TH>iVote Table</TH><TH>Function of Table</TH><TH>Local Name</TH></TR>
  <TR><TD ALIGN=CENTER>Entries</TD><TD ALIGN=CENTER>Contains entry information</TD><TD ALIGN=CENTER>$db_entry</TD></TR>
  <TR><TD ALIGN=CENTER>Voted</TD><TD ALIGN=CENTER>Contains visitor voted IPs</TD><TD ALIGN=CENTER>$db_votes</TD></TR>\n";
  print "</TABLE>\n\n";

  print "<H1>Not implemented yet.</H1>\n";
  print "<A HREF=\"$cgi_url?command=admin\"><IMG SRC=\"$buttons/okay.png\" WIDTH=102 HEIGHT=24 BORDER=0></A><BR><BR>\n";
  &WebFooter;
#  my $sth = $dbh->do( "DELETE * FROM @_" ) ||
#    die "Can't clear table @_:  $DBI::errstr\n";
}


# CreateTables - simply creates a table called "entries" in the $db_name database,
# which must already exist.  Maybe extend to drop/create with 0/1 toggle as a
# "reset function."  Pass "0" to create "entries", "1" for "voted" or "2" for both
#
sub CreateTables  {

  if( @_ ne (0,1,2) )  {
    print "<META HTTP-EQUIV='REFRESH' CONTENT='2; URL=$cgi_url?command=admin'>\n";
    &WebHeader;
    print "<H1>Invalid \"create\" parameters!</H1>\n";
    &WebFooter;
  }

  if( @_ == 0 || @_ == 2 )  {
    my $sth = $dbh->do( "
      CREATE TABLE $db_entry (
      id	INT NOT NULL PRIMARY KEY AUTO_INCREMENT,
      name	VARCHAR(50),
      email 	VARCHAR(50),
      score	SMALLINT,
      blurb	VARCHAR(255),
      entry	CHAR(10))
    " )  ||
      die "Can't create \"$db_entry\" table:  $DBI::errstr\n";
  }

  if( @_ == 1 || @_ == 2 )  {
    my $sth = $dbh->do( "
      CREATE TABLE $db_votes (
      ip	VARCHAR(15) NOT NULL PRIMARY KEY )
    " )  ||
      die "Can't create \"$db_votes\" table:  $DBI::errstr\n";
  }
}


# AddEntry - add an entry into the database.  Very primitive right now
# as you must enter the information manually below.  Must be command
# line and web enabled somehow, if possible.
#
sub AddEntry  {

  my $time    = (localtime);
  my $entered = ($time->year+1900)."-".($time->mon+1)."-".$time->mday;
  my $name    = param('name');
  my $email   = param('email');
  my $blurb   = param('blurb');

  # Enter the submitted information into the database
  #
  my $sth = $dbh->do( "
    INSERT INTO $db_entry VALUES (
      0,
      '$name',
      '$email',
      0,
      '$blurb',
      '$entered')
  " )  ||
  die "Can't add entry:  $DBI::errstr\n";

  print "<META HTTP-EQUIV='REFRESH' CONTENT='2; URL=$cgi_url?command=admin'>\n";
  &WebHeader;
  print "<H1>Database entry added!</H1>\n";
  print "Entry added - returning to admin screen...<BR><BR>\n";
  &WebFooter;
}


# WebDeleteEntry - takes an INT argument for the entry ID to delete
#
sub WebDeleteEntry  {

  # Kill the entry of the incoming ID
  #
  my $sth = $dbh->do( "DELETE FROM $db_entry WHERE id = @_" ) ||
    die "Can't delete entry @_:  $DBI::errstr\n";

  print "<META HTTP-EQUIV='REFRESH' CONTENT='2; URL=$cgi_url?command=admin'>\n";
  &WebHeader;
  print "<H1>Database entry deleted!</H1>\n";
  print "Entry #@_ deleted - returning to admin screen...</CENTER><BR>\n";
  &WebFooter;
}


# WebResetEntry - takes an INT argument for the entry ID of which to reset score
#
sub WebResetEntry  {

  # Reset the score to zero of the incoming ID
  #
  my $sth = $dbh->do( "UPDATE $db_entry SET score = 0 WHERE id = @_" ) ||
    die "Couldn't update database:  $DBI::errstr\n";

  print "<META HTTP-EQUIV='REFRESH' CONTENT='2; URL=$cgi_url?command=admin'>\n";
  &WebHeader;
  print "<H1>Database entry reset!</H1>\n";
  print "Entry #@_ reset - returning to admin screen...</CENTER><BR>\n";
  &WebFooter;
}


# WebAddEntry - Generate Web-based entry addition form
#
sub WebAddEntry  {

  my $voters = 0;
  my $total  = 0;
  
  &WebHeader;
  print "<H1>Add Database Entry</H1>\n";
  print "<FORM METHOD=GET ACTION=\"$cgi_url\">\n";
  print "<INPUT TYPE=HIDDEN NAME=\"command\" VALUE=\"addentry\">\n";
  print "<INPUT TYPE=image IMG SRC=\"$buttons/okay.png\" WIDTH=102 HEIGHT=24 BORDER=0>\n";
  print "<A HREF=\"$cgi_url?command=admin\"><IMG SRC=\"$buttons/cancel.png\" WIDTH=102 HEIGHT=24 BORDER=0></A><BR><BR>\n";
  print "<TABLE CELLSPACING=0 CELLPADDING=2 BORDER=1>\n";
  print "
    <TR><TD>Name<TD><INPUT TYPE=\"TEXT\" NAME=\"name\" WIDTH=\"50\" SIZE=\"50\">
    <TR><TD>Email<TD><INPUT TYPE=\"TEXT\" NAME=\"email\" WIDTH=\"50\" SIZE=\"50\" VALUE=\"\\@\">Escape @ signs!
    <TR><TD>Blurb<TD><TEXTAREA NAME=\"blurb\" COLS=\"63\" ROWS=\"4\" WRAP=\"VIRTUAL\"></TEXTAREA>\n";
  print "</TABLE>\n";
  print "</FORM>\n\n";
  &WebFooter;
}


# WebInterface - Create the Web interface for managing the database
#
sub WebInterface  {

  my $voters      = 0;
  my $total       = 0;
  my $percentage  = 0;
  my $busted      = 0;
  my $busted_flag = 0;
  my $term        = "record";

  # Grab all the scores from the entries and add them up, finally dividing
  # by the number of entries so that we can later calculate percentages...
  #
  my $sth = $dbh->prepare( "SELECT score FROM $db_entry WHERE id >= 0" ) ||
    die "Can't execute SQL statement:  $DBI::errstr\n";
  $sth->execute() ||
    die "Can't execute SQL statement:  $DBI::errstr\n";

  while( my $array_ref = $sth->fetchrow_arrayref )  {
    $voters = $voters + $array_ref->[0];
  }
  die "Fetch failed due to $DBI::errstr" if $DBI::err;

  # Sanity check for an empty table - if not, then figure
  # out how much each vote counts for in percent
  #
  if( $voters > 0 )  {
    $percentage = 100 / $voters;
    $term = "records";
  }  else  {
    $percentage = 0;
  }

  # Can also use "...ORDER BY score DESC" below for descending order
  #
  my $sort = param('sort');
  if( $sort )  {
    $sth = $dbh->prepare( "SELECT id, name, email, score, entry, blurb FROM $db_entry WHERE id > 0 ORDER BY $sort" ) ||
      die "Can't execute SQL statement:  $DBI::errstr\n";
  }  else  {
    $sth = $dbh->prepare( "SELECT id, name, email, score, entry, blurb FROM $db_entry WHERE id > 0 ORDER BY id" ) ||
      die "Can't execute SQL statement:  $DBI::errstr\n";
  }
  $sth->execute() ||
    die "Can't execute SQL statement:  $DBI::errstr\n";

  &WebHeader;
  print "<H1>Database Administration <I>($version)</I></H1>\n
  <I><B>
  Click blurb to view image or delete icon to remove entry<BR>
  Click column header to sort data by that field<BR>
  <A HREF=\"$home/iVote.html\" TARGET=\"_blank\">Documentation for iVote is here</A></B></I><BR><BR>
  <A HREF=\"$cgi_url?command=zap\"><IMG SRC=\"$buttons/zap_table.png\" WIDTH=\"102\" HEIGHT=\"24\" BORDER=\"0\"></A>&nbsp;&nbsp;&nbsp;
  <A HREF=\"$images\"><IMG SRC=\"$buttons/browse.png\" WIDTH=\"102\" HEIGHT=\"24\" BORDER=\"0\"></A>&nbsp;&nbsp;&nbsp;
  <A HREF=\"$cgi_url?command=add\"><IMG SRC=\"$buttons/add_entry.png\" WIDTH=\"102\" HEIGHT=\"24\" BORDER=\"0\"></A>&nbsp;&nbsp;&nbsp;
  <A HREF=\"$cgi_url?$ENV{'QUERY_STRING'}\"><IMG SRC=\"$buttons/refresh.png\" WIDTH=\"102\" HEIGHT=\"24\" BORDER=\"0\"></A>&nbsp;&nbsp;&nbsp;
  <A HREF=\"$cgi_url?command=stats\"><IMG SRC=\"$buttons/stats.png\" WIDTH=\"102\" HEIGHT=\"24\" BORDER=\"0\"></A>&nbsp;&nbsp;&nbsp;
  <A HREF=\"$cgi_url?command=display\"><IMG SRC=\"$buttons/voting.png\" WIDTH=\"102\" HEIGHT=\"24\" BORDER=\"0\"></A><BR><BR>\n\n";
  print "<TABLE WIDTH=\"90%\" CELLSPACING=\"0\" CELLPADDING=\"1\" BORDER=\"1\">\n";
  print "<TR>
    <TH><A HREF=\"$cgi_url?command=admin&sort=id\">ID</A>
    <TH>T/F
    <TH><A HREF=\"$cgi_url?command=admin&sort=name\">Name</A>
    <TH><A HREF=\"$cgi_url?command=admin&sort=email\">Email Address</A>
    <TH><A HREF=\"$cgi_url?command=admin&sort=score+DESC\">Score</A>
    <TH><A HREF=\"$cgi_url?command=admin&sort=entry+DESC\">Entered</A>
    <TH>Res&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Del\n";

  while( my $array_ref = $sth->fetchrow_arrayref )  {
    print "<TR>
      <TD ALIGN=RIGHT>$array_ref->[0]
      <TD ALIGN=CENTER>";

      # Test if the thumbnail and fullsize images exist or not
      # If they do, show green ball - if not, show red one
      #
      if( -e "$file_path/images/t_$array_ref->[0].$image_type" )  {
        print "<IMG SRC=\"$home/green.gif\" BORDER=\"0\" WIDTH=\"12\" HEIGHT=\"12\">";
      } else {
        print "<IMG SRC=\"$home/red.gif\" BORDER=\"0\" WIDTH=\"12\" HEIGHT=\"12\">";
        $busted_flag = 1;
      }
      if( -e "$file_path/images/f_$array_ref->[0].$image_type" )  {
        print "<IMG SRC=\"$home/green.gif\" BORDER=\"0\" WIDTH=\"12\" HEIGHT=\"12\">";
      } else {
        print "<IMG SRC=\"$home/red.gif\" BORDER=\"0\" WIDTH=\"12\" HEIGHT=\"12\">";
        $busted_flag = 1;
      }

      if( $busted_flag )  {
        $busted = $busted + 1;
      }

      print "
      <TD>$array_ref->[1]
      <TD><A HREF=\"mailto:$array_ref->[2]\">$array_ref->[2]</A>
      <TD ALIGN=RIGHT>";
    printf "%3.1f%%&nbsp;&nbsp;(%u)",($array_ref->[3] * $percentage),$array_ref->[3];
    print "
      <TD ALIGN=CENTER>$array_ref->[4]
      <TD ALIGN=CENTER>
      <A HREF=\"$cgi_url?command=res&id=$array_ref->[0]\">
        <IMG SRC=\"$buttons/entry_reset.png\" WIDTH=\"40\" HEIGHT=\"24\" BORDER=\"0\"></A>
      <A HREF=\"$cgi_url?command=del&id=$array_ref->[0]\">
        <IMG SRC=\"$buttons/entry_delete.png\" WIDTH=\"40\" HEIGHT=\"24\" BORDER=\"0\"></A>\n";
    print "<TR>
      <TD COLSPAN=7><FONT SIZE=-1><A HREF=\"$images/f_$array_ref->[0].$image_type\">$array_ref->[5]</A></FONT>\n";
    $total = $total + 1;
  }  
  die "Fetch failed due to $DBI::errstr" if $DBI::err;

  # If there are no entries, suggest that the user should
  # add an entry now, displaying the button else show stats
  #
  if( $total <= 0 )  {
    print "<TR><TD COLSPAN=7 ALIGN=CENTER><BR><H3>You should now go to the <A HREF=\"$cgi_url?command=add\">";
    print "<IMG SRC=\"$buttons/add_entry.png\" WIDTH=\"102\" HEIGHT=\"20\" BORDER=\"0\" ALIGN=\"ABSMIDDLE\"></A> function</H3>\n";
  }  elsif( $voters == 0 )  {
    print "<TR><TD COLSPAN=7 ALIGN=CENTER><BR><H3>There have been no votes yet on $total $term</H3>\n";
  }  else  {
    print "<TR><TD COLSPAN=7 ALIGN=CENTER><BR><H3>Total of $voters votes on $total $term</H3>\n";
  }

  if( $busted )  {
    print "<FONT COLOR=RED>There are one or more broken entries that no longer accumulate votes!</FONT><BR><BR>\n";
  }
  
  print "</TABLE>\n\n<BR>\n\n";
  &WebFooter;
}


# CastVote - Accept the incoming vote and process it, making sure
# that $vote_value is within the bounds of $vote_limit +/-
#
sub CastVote  {

  my $user = $ENV{'REMOTE_ADDR'};
  my $vote_value = param('value');

  # Make sure it's within bounds of $vote_limit +/-
  #
  if( $vote_limit == 0 )  {
    $vote_value = 1;
  }  else {
    if( $vote_value > $vote_limit )  {
      $vote_value = 0;
    }  elsif( $vote_value < (0 - $vote_limit) )  {
      $vote_value = 0;
    }  elsif(  $vote_value == 0 )  {
      $vote_value = 1;
    }
  }

  # If $vote_value is deemed illegitimate (zero) then complain
  #
  if( $vote_value == 0 )  {
    print "<META HTTP-EQUIV='REFRESH' CONTENT='2; URL=$cgi_url?command=display'>\n";
    &WebHeader;
    print "<H1>Your vote value is out of bounds!</H1>\n";
    &WebFooter;
  }

  # Something wrong with user's IP address - spoofing?
  #
  if( $user eq "" )  {
    print "<META HTTP-EQUIV=\"REFRESH\" CONTENT=\"2; URL=$cgi_url?command=display\">\n";
    &WebHeader;
    print "<H1>You need a valid IP to vote!</H1>\n";
    print "(You have \"$user\")\n</CENTER>\n";
    &WebFooter;
  }

  if( $singlevote )  {
    #
    # Grab the user's IP address from the database if possible
    #
    my $sth = $dbh->prepare( "SELECT ip FROM $db_votes WHERE ip = '$user'" ) ||
      die "Couldn't check if $ENV{'REMOTE_ADDR'} already voted:  $DBI::errstr\n";
    my $rv = $sth->execute() ||
      die "Can't execute SQL statement:  $DBI::errstr\n";

    # Gotcha!
    #
    if( $rv == "1" )  {
      print "<META HTTP-EQUIV='REFRESH' CONTENT='2; URL=$cgi_url?command=display'>\n";
      &WebHeader;
      print "<H1>You've already voted from $user!</H1>\n";
      &WebFooter;
      exit(0);
    }  elsif( $rv == "0E0" )  {
      #
      # No problemo!
      #
      my $sth = $dbh->do( "INSERT INTO $db_votes VALUES ( '$user' )" )  ||
        die "Can't add ip $user:  $DBI::errstr\n";
    }
  }

  # Grab the entry's score from the database
  #
  my $sth = $dbh->prepare( "SELECT score FROM $db_entry WHERE id = @_" ) ||
    die "Couldn't fetch score for id #@_ to update:  $DBI::errstr\n";
  $sth->execute() ||
    die "Can't execute SQL statement:  $DBI::errstr\n";

  my @row = $sth->fetchrow_array();

  # Define $vote_limit for maximum per vote change value and after validating
  # above, increment or decrement the vote, but don't go below zero...
  #
  $row[0] = $row[0] + $vote_value;
  if( $row[0] <= 0 )  {
    $row[0] = 0;
  }

  $sth = $dbh->do( "UPDATE $db_entry SET score = $row[0] WHERE id = @_" ) ||
    die "Couldn't update database:  $DBI::errstr\n";

  if( $vote_style == 1 )  {
    print "<META HTTP-EQUIV='REFRESH' CONTENT='2; URL=$cgi_url?command=rankings'>\n";
  }  else  {
    print "<META HTTP-EQUIV='REFRESH' CONTENT='2; URL=$cgi_url?command=display'>\n";
  }
  &WebHeader;
  print "<H1>Your vote has been counted!</H1>\n";
  &WebFooter;
}


# VotePage - Dump out the entries in a nice format for people to vote
# for their favorite entry, allowing thumbnail/full and blurbs
#
sub VotePage  {

  my $colcount  = 1;
  my $total     = 0;
  my $voters    = 0;
  my $busted    = 0;
  my $term      = "entry";
  my $time      = (localtime);
  my $timestamp = ($time->year+1900)."-".($time->mon+1)."-".$time->mday;

  # Grab every entry and all it's details from the database
  # sorted by the ID number of the entry in ascending order
  #
  my $sth = $dbh->prepare( "SELECT id, name, email, blurb, score FROM $db_entry WHERE id >= 0 ORDER BY id" ) ||
    die "Can't execute SQL statement:  $DBI::errstr\n";
  $sth->execute() ||
    die "Can't execute SQL statement:  $DBI::errstr\n";

  &WebHeader;
  
  # Inform visitors of the current situation
  #
  if ( $timestamp => "$cutoff" )  {
    print "<I>Voting for this poll has expired!</I>\n";
  }  elsif( $singlevote == 0 )  {
    print "<I>You may vote multiple times!</I>\n<BR>\n\n";
  }  else  {
    print "<I>You may vote only once!</I>\n<BR>\n\n";
  }

  # This routine prints an ultra-clean voting page with just the
  # images and makes each a link to vote for it.  Observes cutoff
  # and makes sure images exist as per other styles
  #
  if( $vote_style == 1 )  {
    print "<BR>\n\n";
    while( my $array_ref = $sth->fetchrow_arrayref )  {
      #
      # Make sure the image files exist (full-sized version irrelevant)
      #
      if( -e "$file_path/images/t_$array_ref->[0].$image_type" && -e "$file_path/images/f_$array_ref->[0].$image_type" )  {
        #
        # Make sure the poll is still active, otherwise don't enable voting
        #
        if ( $timestamp => "$cutoff" )  {
          print "<IMG SRC=\"$images/t_$array_ref->[0].$image_type\" BORDER=0>\n";
        }  else  {
          print "<A HREF=\"$cgi_url?command=vote&id=$array_ref->[0]\"><IMG SRC=\"$images/t_$array_ref->[0].$image_type\" BORDER=0></A>\n";
        }
        $colcount = $colcount +1;
        if( $colcount == 2 )  {
          $colcount = 0;
	  print "&nbsp;&nbsp;&nbsp;";
        }  else  {
          print "<BR><BR>\n";
        }
      }
    }
    if( $colcount == 0 )  {
      print "<BR><BR>\n";
    }
    &WebFooter;
  }
  
  print "<TABLE CELLSPACING=\"10\" CELLSPACING=\"0\" BORDER=\"0\">\n";

  while( my $array_ref = $sth->fetchrow_arrayref )  {
    if( $colcount == 1 )  {
      print "<TR>\n";
    }

    # Unless both thumbnail and full-sized images exist, do not display entry
    #
    if( -e "$file_path/images/t_$array_ref->[0].$image_type" && -e "$file_path/images/f_$array_ref->[0].$image_type" )  {
    print "<TD>
    <TABLE WIDTH=\"250\" CELLSPACING=\"0\" CELLPADDING=\"5\" BORDER=\"1\">
    <TR>
      <TD ALIGN=CENTER>$array_ref->[4]</TD>\n";

    # Check to see if names should be hyperlinked via a "mailto:"
    #
    if( $mailto_name == 1 )  {
      print "<TD ALIGN=CENTER><A HREF=\"mailto:$array_ref->[2]\">$array_ref->[1]</A></TD>\n";
    }  else  {
      print "<TD ALIGN=CENTER>$array_ref->[1]</TD>\n";
    }

    # Is the cutoff active?  If it's not, go into processing the voting buttons
    #
    if ( $timestamp => "$cutoff" )  {
      print "<TD ALIGN=CENTER>&nbsp;</TD>\n";
    } else {
      #
      # Code for auto-detection of ranked voting and automatic voting button generation
      #
      if( $vote_limit == 2 )  {
        print "<TD ALIGN=CENTER><A HREF=\"$cgi_url?command=vote&id=$array_ref->[0]&value=-2\">-</A>&nbsp;\n";
        print "<TD ALIGN=CENTER><A HREF=\"$cgi_url?command=vote&id=$array_ref->[0]&value=-1\">-</A>&nbsp;/&nbsp;\n";
        print "<TD ALIGN=CENTER><A HREF=\"$cgi_url?command=vote&id=$array_ref->[0]&value=1\">+</A>&nbsp;\n";
        print "<TD ALIGN=CENTER><A HREF=\"$cgi_url?command=vote&id=$array_ref->[0]&value=2\">+</A></TD>\n";
      }  elsif( $vote_limit == 1 )  {
        print "<TD ALIGN=CENTER><A HREF=\"$cgi_url?command=vote&id=$array_ref->[0]&value=-1\">-</A>&nbsp;/&nbsp;\n";
        print "<TD ALIGN=CENTER><A HREF=\"$cgi_url?command=vote&id=$array_ref->[0]&value=1\">+</A></TD>\n";
      }  else  {
        print "<TD ALIGN=CENTER><A HREF=\"$cgi_url?command=vote&id=$array_ref->[0]\">VOTE!</A></TD>\n";
      }
    }

    print "
    </TR>
    <TR>
      <TD ALIGN=CENTER COLSPAN=3 BGCOLOR=BLACK><A HREF=\"$images/f_$array_ref->[0].$image_type\"><IMG SRC=\"$images/t_$array_ref->[0].$image_type\" BORDER=0></A></TD>
    </TR>\n";

    # Check to see if blurbs are to be printed...
    #
    if( $blurb == 1 )  {
      print "<TR>\n
        <TD ALIGN=CENTER COLSPAN=3><FONT SIZE=-1>$array_ref->[3]</FONT></TD>
      </TR>\n";
    }

    print "</TABLE>\n</TD>\n";
    $voters = $voters + $array_ref->[4];
    $total = $total + 1;
    $colcount = $colcount + 1;
    } else {
      $busted = $busted + 1;
    }
    
    if( $colcount == 2 )  {
      $colcount = 0;
    }
  }
  die "Fetch failed due to $DBI::errstr" if $DBI::err;

  print "</TABLE>\n\n";

  # If there are no entries, suggest that the user should
  # add an entry now, displaying the button else show stats
  #
  if( $total <= 0 && $voters == 0 )  {
    print "<H3>You should now go to the <A HREF=\"$cgi_url?command=add\"><IMG SRC=\"$buttons/add_entry.png\" WIDTH=\"102\" HEIGHT=\"24\" BORDER=\"0\" ALIGN=\"ABSMIDDLE\"></A> function</H3>\n";
  } elsif( $total <= 0 && $voters >= 1 )  {
    print "<H3>There are broken entries in the database</H3>\n";
  } elsif( $voters == 0 )  {
    print "<H3>There have been no votes yet on $total $term</H3>\n";
  }  else  {
    $term = "entries";
    print "<H3>Total of $voters votes on $total $term</H3>\n";
  }
  &WebFooter;
  exit(0);
}


# StatsPage - Dump a mass of information about the database, variables, etc.
#
sub StatsPage  {

  my $colcount = 1;
  my $total    = 0;
  my $voters   = 0;
  my $busted   = 0;

  my $sth = $dbh->prepare( "SELECT id, score FROM $db_entry WHERE id > 0" ) ||
    die "Can't execute SQL statement:  $DBI::errstr\n";
    
  $sth->execute() ||
    die "Can't execute SQL statement:  $DBI::errstr\n";

  print "
<SCRIPT language=\"JavaScript\">
<!--
function LiveStats() {
window.open('$cgi_base/livestats.pl','EANITHING','toolbar=no,location=no,directories=no,status=no,menubar=no,resizable=yes,copyhistory=no,scrollbars=yes,width=180,height=100,top=100,left=200');
}
//-->
</SCRIPT>\n";

  &WebHeader;
  print "<H1>Current Statistics</H1>\n";
  print "<A HREF=\"$cgi_url?command=admin\"><IMG SRC=\"$buttons/okay.png\" WIDTH=102 HEIGHT=24 BORDER=0></A><BR><BR>\n";
  print "<TABLE CELLSPACING=\"10\">\n<TR>\n";

  while( my $array_ref = $sth->fetchrow_arrayref )  {
    if( -e "$file_path/images/t_$array_ref->[0].$image_type" && -e "$file_path/images/f_$array_ref->[0].$image_type" )  {
      $total = $total + 1;
    } else {
      $busted = $busted + 1;
    }
    $voters = $voters + $array_ref->[1];
  }

  print "<TABLE CELLSPACING=\"0\" CELLPADDING=\"3\" BORDER=\"1\">
  <TR>
    <TH COLSPAN=\"2\">Local Environment Parameters
  <TR>
    <TD>iVote doc root
    <TD ALIGN=LEFT>$home
  <TR>
    <TD>iVote CGI base
    <TD ALIGN=LEFT>$cgi_base
  <TR>
    <TD>iVote CGI URL
    <TD ALIGN=LEFT>$cgi_url
  <TR>
    <TD>Interface images
    <TD ALIGN=LEFT>$buttons
  <TR>
    <TD>Entry images
    <TD ALIGN=LEFT>$images
  <TR>
    <TD>Page title
    <TD><FONT SIZE=-1>$dump_page_title</FONT>
  <TR>
    <TD>Page attributes
    <TD><FONT SIZE=-1>$dump_page_attrs</FONT>
  <TR>
    <TD>Header document
    <TD><FONT SIZE=-1>$header_doc</FONT>
  <TR>
    <TD>Footer document
    <TD><FONT SIZE=-1>$footer_doc</FONT>
  <TR>
    <TD>Image type
    <TD>$image_type
  <TR>
    <TD>Allow deleting
    <TD ALIGN=\"LEFT\">";
  if( $allowdelete == 1 )  {
    print "Yes\n";
  }  else  {
    print "No\n";
  }
  print "
  <TR>
    <TD>Multi-vote
    <TD ALIGN=\"LEFT\">";
  if( $singlevote == 0 )  {
    print "Yes\n";
  }  else  {
    print "No\n";
  }
  print "</TABLE>\n\n";
  print "<TABLE CELLSPACING=\"0\" CELLPADDING=\"3\" BORDER=\"0\">\n<TR>
  <TD>
  <TABLE CELLSPACING=\"0\" CELLPADDING=\"4\" BORDER=\"1\">
  <TR>
    <TH COLSPAN=\"2\">iVote Information</TH>
  <TR>
    <TD>iVote version</TD>
    <TD>$version</TD>
  <TR>
    <TD>Mod_perl enabled</TD>
    <TD ALIGN=\"LEFT\">";
  if( $ENV{'MOD_PERL'} )  {
    print "Yes: $ENV{'MOD_PERL'}</TD>\n";
  }  else  {
    print "No</TD>\n";
  }
  print "  </TABLE>\n  </TD>
  <TD>
  <TABLE CELLSPACING=\"0\" CELLPADDING=\"4\" BORDER=\"1\">
  <TR>
    <TH COLSPAN=\"2\"><A HREF=\"javascript:LiveStats()\">Database Statistics</A>
  <TR>
    <TD>Total <A HREF=\"$cgi_url?command=lvoted\">voters</TD>
    <TD ALIGN=CENTER><B>$voters</B></TD>
  <TR>
    <TD>Total entries</TD>
    <TD ALIGN=CENTER><B>$total</B> <FONT COLOR=RED>($busted)</FONT></TD>
  </TABLE>\n  </TD>\n</TR>\n</TABLE>\n\n<BR>\n";

# Inline notes for possible use in realtime stats
#
#  open (OUT, "/usr/local/mysql/bin/mysqladmin status |") ||
#    die "Can't open mysqladmin binary\n";
#  my $status=<OUT>;
#  $status =~ s/\013/<BR>/g;
#
#  print "<TABLE CELLSPACING=\"0\" CELLPADDING=\"5\" BORDER=\"1\">\n<TR>\n<TH>Database Server
#  <TR>
#    <TD><PRE>$status</PRE>
#  </TABLE>\n";

  &WebFooter;
}


# Rankings - Print a nice, compact and embeddable "rankings" display,
# which is useful for incorporation into an existing Website or page
#
sub Rankings  {

  my $count = 0;

  my $sth = $dbh->prepare( "SELECT id, name, score FROM $db_entry WHERE id > 0 ORDER BY score DESC" ) ||
    die "Can't execute SQL statement:  $DBI::errstr\n";

  my $rv = $sth->execute() ||
    die "Can't execute SQL statement:  $DBI::errstr\n";

  if( $rank_head )  {
    &WebHeader;
  }

  print "<TABLE CELLPADDING=\"5\" CELLSPACING=\"0\" BORDER=\"1\">\n";
  print "<TR>\n  <TH COLSPAN=\"2\">Top $max_rank Entries\n";

  # Print out each entry's name and score until we hit $max_rank
  #
  while( my $array_ref = $sth->fetchrow_arrayref )  {
    print "<TR>\n  <TD>$array_ref->[1]\n  <TD ALIGN=\"CENTER\">$array_ref->[2]\n";
    $count++;
    if( $count == $max_rank )  {
      last;
    }
  }  
  die "Fetch failed due to $DBI::errstr" if $DBI::err;

  print "<TR>\n  <TD COLSPAN=\"2\" ALIGN=\"CENTER\">";
  print "<A HREF=\"http://eunuchs.org/linux/ivote/index.html\">iVote</A> &copy 2001\n";
  print "</TABLE>\n</CENTER>\n\n</BODY>\n</HTML>";

  # If $rank_next is defined, insert the okay button with that URL link
  #
  if( $rank_next ne "" )  {
    print "<BR>\n<CENTER>\n  ";
    print "<A HREF=\"$rank_next\">\n<IMG SRC=\"$buttons/okay.png\" WIDTH=102 HEIGHT=24 BORDER=0>\n";
    print "</A>\n</CENTER>\n";
  }
}


# ListVoted - Print a nice, compact and embeddable "rankings" display,
# which is useful for incorporation into an existing Website or page
#
sub ListVoted  {

  my $count = 0;
  my $colcount = 0;

  my $sth = $dbh->prepare( "SELECT * FROM $db_votes" ) ||
    die "Can't execute SQL statement:  $DBI::errstr\n";

  my $rv = $sth->execute() ||
    die "Can't execute SQL statement:  $DBI::errstr\n";

  &WebHeader;
  print "<H1>IP Addresses currently registered</H1>\n";
  print "<A HREF=\"$cgi_url?command=stats\"><IMG SRC=\"$buttons/okay.png\" WIDTH=102 HEIGHT=24 BORDER=0></A><BR><BR>\n";

  # Print out each IP address in $db_votes, four across
  #
  print "<TABLE BORDER=\"0\" CELLSPACING=\"5\" CELLPADDING=\"5\">\n<TR>\n";
  while( my $array_ref = $sth->fetchrow_arrayref )  {
    print "  <TD>$array_ref->[0]</TD>\n";
    $count++;
    $colcount++;
    if( $colcount == 4 )  {
      $colcount = 0;
      print "</TR>\n<TR>\n";
    }
  }
  die "Fetch failed due to $DBI::errstr" if $DBI::err;

  if( $count == 0 )  {
    print "</TABLE>\n<B>Currently no IP addresses registered</B>\n\n<BR><BR>\n";
  }  elsif( $count == 1 )  {
    print "</TABLE>\n<B>One IP addresse registered</B>\n\n<BR><BR>\n";
  }  else  {
    print "</TABLE>\n<B>Total of $count IP addresses registered</B>\n\n<BR><BR>\n";
  }
  &WebFooter;
}


#
# HERE IS THE MAIN LOOP AND WHERE EXECUTION BEGINS...
#


# Whatever happens, we're going to be sending out HTML
# to the client, so might as well make it neat...
#
&CGI::PrintHeader;
print "<HTML>\n<HEAD>\n<TITLE>$dump_page_title</TITLE>\n";


# Check to see if iVote is down and exit if it is; useful if you have database
# or other issues as this checks and exits before anything else...
#
if( $ivote_down == 1 )  {
  &WebHeader;
  print "<H1>System is temporarily down!</H1>\n";
  print "Please try again in a little while.<BR><BR>\n";
  &WebFooter;
  exit(0);
}


# Open up the database connection (MODIFY USER/PASS HERE!)
#
$dbh = DBI->connect( "DBI:mysql:$db_name", "root", "" )  ||
  die "Can't connect to database:  $DBI::errstr\n";


# Process the command passed from the query string
#
if( param('command') eq "admin" )  {             # Administration screen
  &WebInterface;
}  elsif( param('command') eq "add" )  {         # Actual add entry CGI
  &WebAddEntry;
}  elsif( param('command') eq "addentry" )  {    # Web form to add entry
  &AddEntry;
}  elsif( param('command') eq "del" )  {         # Delete an entry
  if( $allowdelete == 1 )  {
    &WebDeleteEntry( param('id') );
  } else {
    &NoDeleteReset;
  }
}  elsif( param('command') eq "res" )  {         # Reset an entry
  if( $allowdelete == 1 )  {
    &WebResetEntry( param('id') );
  } else {
    &NoDeleteReset;
  }
}  elsif( param('command') eq "vote" )  {        # Vote for an entry
  &CastVote( param('id') );
}  elsif( param('command') eq "create" )  {      # Create table
  &CreateTables( param('table') );
}  elsif( param('command') eq "zap" )  {         # Zap/clear table
  &ZapDB( param('table') );
}  elsif( param('command') eq "stats" )  {       # Dump statistics information
  &StatsPage;
}  elsif( param('command') eq "display" )  {     # Show user vote page
  &VotePage;
}  elsif( param('command') eq "rankings" )  {    # Show rankings
  &Rankings;
}  elsif( param('command') eq "lvoted" )  {      # List voted table
  &ListVoted;
}  else  {                                       # Default value, no parameters
  &VotePage;
}


# Close down the database connection
#
#$dbh->disconnect ||
#  warn "Disconnection failed:  $DBI::errstr\n";

# All's well that ends well...
#
exit(0);
